﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.IO.Ports;
using System.Collections;

namespace mbed_Fitbit
{
    public partial class Form1 : Form
    {
        string selectedPort;
        string[] date = new string[7];
        int[] steps = new int[7];
        int[] floors = new int[7];
        double[] calories = new double[7];
        double[] distance = new double[7];
        int count = 0;

        public Form1()
        {
            InitializeComponent();
        }


        private void Form1_Load(object sender, EventArgs e)
        {
            string[] ports = SerialPort.GetPortNames();
            comboBox1.Items.AddRange(ports);
            //Disable load and plot buttons as no data stored
            loadBtn.Enabled = false;
            stepPlotBtn.Enabled = false;
            floorsPlotBtn.Enabled = false;
            caloriePlotBtn.Enabled = false;
            distancePlotBtn.Enabled = false;
        }

        private void connectBtn_Click(object sender, EventArgs e)
        {
            try
            {
                selectedPort = comboBox1.SelectedItem.ToString();
                //Set up serial communication with mbed
                serialPort1.PortName = selectedPort;
                serialPort1.ReadTimeout = 500;
                loadBtn.Enabled = true;
                statusLabel.Text = "Connected to " + selectedPort;
                serialPort1.Open();

            }
            catch (Exception ex)
            {
                statusLabel.Text = "Error";
                MessageBox.Show(ex.Message.ToString(), "Error!", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void loadBtn_Click(object sender, EventArgs e)
        {
            serialPort1.DiscardInBuffer();
            serialPort1.DataReceived += new SerialDataReceivedEventHandler(serialPort1_DataReceived);
            stepPlotBtn.Enabled = true;
            floorsPlotBtn.Enabled = true;
            caloriePlotBtn.Enabled = true;
            distancePlotBtn.Enabled = true;
            string message = "!p";
            serialPort1.WriteLine(message);
            count = 0;
            chart1.ChartAreas[0].AxisX.Title = "Date (month/day/year)";
        }

        private void serialPort1_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            try
            {
                string indata = serialPort1.ReadLine();
                Console.WriteLine(indata);
                string[] data = indata.Split('\t');
                Console.WriteLine($"<{data}>");
                //Receive and parse through data line from mbed
                //Store data in arrays
                if (count < 7)
                {
                    date[count] = data[0];
                    steps[count] = Int32.Parse(data[1]);
                    floors[count] = Int32.Parse(data[2]);
                    calories[count] = Math.Round(double.Parse(data[3]), 2);
                    distance[count] = Math.Round(double.Parse(data[4]), 2);
                    count++;
                }
            }
            catch (TimeoutException) { }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message.ToString(), "Error!", MessageBoxButtons.OK, MessageBoxIcon.Error);
                serialPort1.DataReceived -= new SerialDataReceivedEventHandler(serialPort1_DataReceived);
            }
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            serialPort1.DataReceived -= new SerialDataReceivedEventHandler(serialPort1_DataReceived);
            serialPort1.Close();
        }

        private void plotBtn_Click(object sender, EventArgs e)
        {
            chart1.Titles[0].Text = "Steps";
            chart1.ChartAreas[0].AxisY.Title = "Steps";
            chart1.Series["Steps"].Enabled = true;
            chart1.Series["Floors"].Enabled = false;
            chart1.Series["Calories"].Enabled = false;
            chart1.Series["Distance"].Enabled = false;
            chart1.Series["Steps"].Points.Clear();
            for (int i = 0; i < date.Length; i++)
            {
                if (date[i] == null)
                {
                    date[i] = "None";
                }
                chart1.Series["Steps"].Points.AddXY(date[i], steps[i]);
            }
        }

        private void floorsPlotBtn_Click(object sender, EventArgs e)
        {
            chart1.Titles[0].Text = "Flights Climbed";
            chart1.ChartAreas[0].AxisY.Title = "Floors";
            chart1.Series["Steps"].Enabled = false;
            chart1.Series["Floors"].Enabled = true;
            chart1.Series["Calories"].Enabled = false;
            chart1.Series["Distance"].Enabled = false;
            chart1.Series["Floors"].Points.Clear();
            for (int i = 0; i < date.Length; i++)
            {
                if (date[i] == null)
                {
                    date[i] = "None";
                }
                chart1.Series["Floors"].Points.AddXY(date[i], floors[i]);
            }

        }

        private void caloriePlotBtn_Click(object sender, EventArgs e)
        {
            chart1.Titles[0].Text = "Calories Burned";
            chart1.ChartAreas[0].AxisY.Title = "Calories";
            chart1.Series["Steps"].Enabled = false;
            chart1.Series["Floors"].Enabled = false;
            chart1.Series["Calories"].Enabled = true;
            chart1.Series["Distance"].Enabled = false;
            chart1.Series["Calories"].Points.Clear();
            for (int i = 0; i < date.Length; i++)
            {
                if (date[i] == null)
                {
                    date[i] = "None";
                }
                chart1.Series["Calories"].Points.AddXY(date[i], calories[i]);
            }
        }

        private void distancePlotBtn_Click(object sender, EventArgs e)
        {
            chart1.Titles[0].Text = "Walking Distance";
            chart1.ChartAreas[0].AxisY.Title = "Distance (ft)";
            chart1.Series["Steps"].Enabled = false;
            chart1.Series["Floors"].Enabled = false;
            chart1.Series["Calories"].Enabled = false;
            chart1.Series["Distance"].Enabled = true;
            chart1.Series["Distance"].Points.Clear();
            for (int i = 0; i < date.Length; i++)
            {
                if (date[i] == null)
                {
                    date[i] = "None";
                }
                chart1.Series["Distance"].Points.AddXY(date[i], distance[i]);
            }
        }

        private void infoBtn_Click(object sender, EventArgs e)
        {
            string infoMessage = "This GUI allows you to view your workout activity data tracked by mbed Fitness Tracker over the past week.\n" +
                "1. Connect to the mbed by selecting its COM Port and clicking the connect button.\n" +
                "2. Click the load button and mbed will send all the activity data to be plotted.\n" +
                "3. Wait two to three seconds to allow mbed to send over all the data\n" +
                "4. Click the steps, floors, calories, and distance buttons on the right to display the appropriate data.";
            MessageBox.Show(infoMessage, "Info", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        private void comboBox1_Click(object sender, EventArgs e)
        {
            string[] ports = SerialPort.GetPortNames();
            comboBox1.Items.Clear();
            comboBox1.Items.AddRange(ports);
        }
    }
}
